//--------------------------------------------------------//
//                                                        //
//   File:    COLORMAP.CPP                                //
//                                                        //
//   Desc:    Palette modification and color mapping      //
//            functions.                                  //
//                                                        //
//--------------------------------------------------------//
#include <alloc.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include "metawndo.h"      /* master MetaWINDOW include file */
#include "GUIconst.h"
#include "GUIextrn.h"
#include "colormap.hpp"
extern  palData   PCXColors[256];

int *clrmap;
rgb *imgpal;
extern rect *scrnRect;
extern "C" {

long cdecl dist3( int x1, int x2, int y1, int y2, int z1, int z2 );

}

// use this dist3 if compiling in 32 bit protected mode
/*
long dist3( int r1, int r2, int g1, int g2, int b1, int b2 )
{
	     r1=r1-r2;
	     r1=r1*r1;
	     g1=g1-g2;
	     g1=g1*g1;
	     b1=b1-b2;
	     b1=b1*b1;
	     return(r1+g1+b1);

}
*/
void show_core(char *txt)
{
   unsigned long avail;

char mess[80];
	avail=(unsigned long)	coreleft();
	TextAlign( alignRight, alignBaseline );
	 PenColor(-1);
	sprintf(mess, "%s Memory = %lu",txt, avail);
  MoveTo(scrnRect->Xmax/2,scrnRect->Ymax/2);
 DrawString(mess);
  MoveTo(scrnRect->Xmax/2,50+scrnRect->Ymax/2);
 DrawString("press any to continue");

	KeyEvent( True, &opEvent );

}

void Pal_Table(void)
{
   // allocate and initialize palettes
   // if image is palette based
   rgb *devpal;
int  nclrs=256,devcnt,i;
    palData   RGBdata[256];

	imgpal = new rgb [nclrs];
	if( imgpal == 0 )   return;

	devpal = new rgb [nclrs];
	if( devpal == 0 )   return;
	 clrmap = new int [nclrs];
	if( clrmap == 0 ) return;

	for( i=0; i<nclrs; i++ )
	{
	   clrmap[i] = i;
	}
	// initialize device palette to image palette
		 for( i=0; i<256; i++ )
   {
	    PCXColors[i].palRed /=256;
	    PCXColors[i].palGreen /= 256;
	    PCXColors[i].palBlue /= 256;

	    imgpal[i].red = (unsigned char)PCXColors[i].palRed;
	    imgpal[i].grn = (unsigned char)PCXColors[i].palGreen;
	    imgpal[i].blu = (unsigned char)PCXColors[i].palBlue;
   }


	for( i=0; i<256; i++ ){  devpal[i] = imgpal[i];}

    reduce_palette(devpal,256,16);
    color_map(imgpal,256,devpal,16,clrmap,2);

	for( i=0; i<nclrs; i++ )
   {

	 RGBdata[i].palRed=devpal[i].red;
	RGBdata[i].palGreen=devpal[i].grn;
	RGBdata[i].palBlue= devpal[i].blu;
	RGBdata[i].palRed   *= 256;
	RGBdata[i].palGreen *= 256;
	RGBdata[i].palBlue  *= 256;

	 }
  WritePalette( 0, 0,nclrs-1 , RGBdata );
    delete imgpal;
    delete devpal;
}


//..................swap two palette entries

void xchg_rgb( rgb *pal, int m, int n )
{
   rgb tmp = pal[m];
   pal[m] = pal[n];
   pal[n] = tmp;
}

//..................determine closest rgb in a list

int closest_rgb( rgb x, rgb *pal, int npal )
{
   long dmin = dist3( x.red, pal[0].red,
			    x.grn, pal[0].grn,
			    x.blu, pal[0].blu );
   int closest = 0;

   for( int i=1; i<npal; i++ )
   {
	long d = dist3( x.red, pal[i].red,
                      x.grn, pal[i].grn,
                      x.blu, pal[i].blu );
      if( d < dmin )
      {
         dmin = d;
         closest = i;
      }
   }

   return closest;
}

//..................find rgb in mid..max maximal to 0..mid-1

int maximal_rgb( rgb *pal, int mid, int max )
{
   long n = 1L << 30;  // a large number
   long dmin = n;
   int  maximal = mid;
   for( int i=mid; i<max; i++ )
   {
      long dsum = 0;
      for( int j=0; j<mid; j++ )
      {
         long d = dist3( pal[i].red, pal[j].red,
                         pal[i].grn, pal[j].grn,
                         pal[i].blu, pal[j].blu );
         if( d == 0 )
         {
             dsum = n;
             break;
         }
         dsum += n / d;
      }
      if( dsum < dmin )
      {
         dmin = dsum;
         maximal = i;
      }
   }
   return maximal;
}

//..................compute maximally apart reduced palette

void reduce_palette( rgb *pal, int npal, int nsub )
{
   // find "blackest" entry as a starting point
   int  i = closest_rgb( rgb(0,0,0), pal, npal );

   // reorder palette so that first nsub entries
   // are maximally far apart
   int imin = 0;
   while( imin < nsub )
   {
      xchg_rgb( pal, i, imin++ );
      i = maximal_rgb( pal, imin, npal );
   }
}

//..................compute color map, optionally smooth

void color_map( rgb *img, int nimg, rgb *dev, int ndev,
                int *cmap, int smooth_wt )
{
   int  *ccnt = 0;
   long *rsum = 0,
        *gsum = 0,
        *bsum = 0;

   // smooth_wt > 0 enables averaging filter
   {
      ccnt = new int [ndev];
      rsum = new long [ndev*3];
      if( (ccnt == 0) || (rsum == 0) )
      {
         smooth_wt = 0;
         if( ccnt ) delete ccnt;
         if( rsum ) delete rsum;
      }
      else
      {
         gsum = rsum + ndev;
         bsum = gsum + ndev;
         // initialize weighted sums
         int n = smooth_wt - 1;
         for( int i=0; i<ndev; i++ )
         {
            rsum[i] = dev[i].red;  rsum[i] *= n;
            gsum[i] = dev[i].grn;  gsum[i] *= n;
            bsum[i] = dev[i].blu;  bsum[i] *= n;
            ccnt[i] = n;
         }
      }
   }

   // compute the color map
   for( int i=0; i<nimg; i++ )
   {
      int j = closest_rgb( img[i], dev, ndev );
      cmap[i] = j;
      if( smooth_wt > 0 )
      {
         ccnt[j] += 1;
         rsum[j] += img[i].red;
         gsum[j] += img[i].grn;
         bsum[j] += img[i].blu;
      }
   }

   // smooth the device palette
   if( smooth_wt > 0 )
   {
      for( i=0; i<ndev; i++ )
      {
         rsum[i] /= ccnt[i];
         dev[i].red = rsum[i];
         gsum[i] /= ccnt[i];
         dev[i].grn = gsum[i];
         bsum[i] /= ccnt[i];
         dev[i].blu = bsum[i];
      }
   }

   if( ccnt ) delete ccnt;
   if( rsum ) delete rsum;
}
