#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <direct.h>
#include <io.h>
#include <dos.h>
#include "metawndo.h"
#include "GUIconst.h"
#include "GUIextrn.h"


static dialogBox  fileDlgBox;
static scrollBar fileSBar;

static dirRec *entries;
static dirRec *entrptr;
void ScrollFiles( int );
void DisplayFiles( int, dirRec * );
void DisplaySpec( char *, char * );
int SortDir( const void *, const void * );
int SelectFile( int, int, rect *, int );
void SplitSpec( char *, char * );

int FileDialog( char *filespec )
/*
Do a file dialog box. Start in current directory with filespec.
Returns ok or cancel, sets filespec to entered path and spec.
*/
{

   dialogBox *dia;
   int   x,y, i, numentries, eventType, retcode;
   int  done, reread, numleft, okenabled;
   unsigned int clickTime;
   char tmpspec[MAXLEN], path[MAXLEN], *p;
   rect tR, fR;
   point DiaCtr;
   int DiaLf, DiaTop, DiaRt, DiaBot;  


   /* center dialog box on screen */
   DiaCtr.X = scrnPort->portRect.Xmax / 2;
   DiaCtr.Y = (scrnPort->portRect.Ymax / 2)-picY(25);

   /* set rects for file dialog box */

   /* compute edges if 4.00 x 5.00 inches */
   DiaLf = DiaCtr.X - picX(200);
   DiaRt = DiaCtr.X + picX(200);
   DiaTop = DiaCtr.Y - picY(250);
   DiaBot = DiaCtr.Y + picY(250);

   SetRect(&fileDlgBox.BoundRect, DiaLf,DiaTop,DiaRt,DiaBot );

   /* place OK button */
   SetRect(&fileDlgBox.OkR,  DiaLf + picX(25), DiaBot - picY(65),
			     DiaLf + picX(125), DiaBot - picY(30) );

   /* place cancel button */
   SetRect(&fileDlgBox.CanR, DiaRt - picX(25), DiaBot - picY(65),
                             DiaRt - picX(125), DiaBot - picY(30) );

   /* place file rect */
   SetRect(&fileDlgBox.BoxR, DiaLf + picX(25), DiaTop + picY(110),
                             DiaRt - picX(50), DiaBot - picY(80) );

   /* place text rect */
   SetRect(&fileDlgBox.TxR,  DiaLf + picX(25), DiaTop + picY(20),
			     DiaRt - picX(25), DiaTop + picY(55) );

   /* place key entry rect */
   SetRect(&fileDlgBox.KeyR,  DiaLf + picX(25), DiaTop + picY(60),
                              DiaRt - picX(25), DiaTop + picY(95) );

   /* set up file dialog scroll bar */
   DupRect( &fileDlgBox.BoxR, &fileSBar.SBRect );
   fileSBar.SBRect.Xmin = fileSBar.SBRect.Xmax + 1;
   fileSBar.SBRect.Xmax += picX( 25 );
   fileSBar.minval = fileSBar.curval = 0;
   fileSBar.maxval = 100;

   /* pop up the box */
   dia = &fileDlgBox;
   ShowDialog( diaFile, dia );

   /* draw in the scroll bar */
   DrawScrollBar( &fileSBar );

   /* init function pointer for scroll bar */
   fileSBar.sbfunc = ScrollFiles;

   /* start double click time */
   PeekEvent( 1, &opEvent );
   clickTime = opEvent.eventTime;

   /* get current path  */
   getcwd( path, MAXLEN );

   /* get the starting spec */
   strcpy( tmpspec, filespec );

   entries = entrptr = NULL;
   reread = True;
   done = False;


   while( !done ) {

      /* re-read directory entries matching input spec and display */
      if( reread ) {

         /* this may take a while */
         CursorStyle(  watchCurs );

         /* no inverted file box */
         fR.Xmin = -1;

         /* enable ok button if disabled */
         DrawButton( &dia->OkR, "Ok" );
	 okenabled = True;

         /* deallocate memory if previously allocated */
         if( entries != NULL )
            free( entries );
         entrptr = NULL;


	 /* split up file path and file spec */
	 SplitSpec( path, tmpspec );

         /* build an absoulte filespec from path and tmpspec */
         sprintf( filespec, "%s%s", path, tmpspec );

         DisplaySpec( path, tmpspec ); 

         /* check number of matching entries */
	 numleft = numentries = FileQuery( filespec, NULL, 0 );

         /* if have any */
	 if( numentries ) {

            /* try to allocate memory for them */
	    entries =(dirRec *) malloc( numentries * sizeof( dirRec ) );
            if( entries == NULL ) {
	       DisplayFiles( numentries, NULL );
               CursorStyle( curCursor ); 
               reread = False;
               continue;
	    }

            /* read them into memory */
            FileQuery( filespec, entries, numentries );

            /* sort them */
            qsort( (void *) entries, (size_t) numentries,
                    sizeof( dirRec ), SortDir );
	 }


	 /* display them */
         DisplayFiles( numentries, entries );

         /* set limits for scroll bar (base at 0) */
         fileSBar.minval = fileSBar.curval = 0;
         fileSBar.maxval = numentries - 1;
         if( fileSBar.maxval < 0 )
            fileSBar.maxval = 0;

	 PositionSB( &fileSBar );

         /* disable ok button if not just one entry */
         if( numentries != 1 ) {
            DialogButton( &dia->OkR, -1 );
            okenabled = False;
         }

	 CursorStyle( curCursor );

	 reread = False;

      }   /* end of if reread */
      
      /* wait for an event */
      KeyEvent( True, &opEvent );
      eventType = opEvent.eventType;
      x = opEvent.eventX; y = opEvent.eventY;

      /* determine what action(s) necessary */
      switch( eventType ) {

         case evntPRESS:

            retcode = -1;

            if( XYInRect(x,y,&dia->KeyR) ) {
	       /* entering a new file spec */
	       strcpy( filespec, tmpspec );
               retcode = DialogText( diaFile, dia, tmpspec );
               if( retcode == Cancel )
		  strcpy( tmpspec, filespec );
               reread = True;
            }
            if( XYInRect(x,y,&dia->OkR) && okenabled ) {
               /* accept file selected */
               retcode = Ok;
               done = True;
            }
            if( XYInRect(x,y,&dia->CanR ) ) {
	       /* cancel everything */
               retcode = Cancel;
               done = True;
            }

            /* if a button pressed */
            if( retcode == Ok || retcode == Cancel ) {

	       /* turn button on */
               if( retcode == Ok )
                  DupRect( &dia->OkR, &tR );
	       else
                  DupRect( &dia->CanR, &tR );

               OnButton( &tR );

               break;
            }

            if( XYInRect(x,y,&dia->BoxR ) ) {
	       /* select a file */

               i = SelectFile( x, y, &fR, numleft );
               if( i < 0 )
                  break;
                  
               p = (entrptr + i)->fileName;

	       /* if a dir selected, path to it and *.*  */
               if( (entrptr + i)->fileAttr & 0x10 ) {
                  sprintf( tmpspec, "%s\\*.*", p );
		  reread = True;
                  break;
               }

               /* else just display the spec */
               strcpy( tmpspec, p );
               sprintf( filespec, "%s%s", path, tmpspec );
	       DisplaySpec( path, tmpspec );

	       /* enable ok button */
               DrawButton( &dia->OkR, "Ok" );
               okenabled = True;

               /* check double click */
               if( ( (unsigned) opEvent.eventTime - clickTime ) < dblClick ) {
                  retcode = Ok;
                  done = True;

                  /* turn OK button on a moment */
                  DialogButton( &dia->OkR, True );
		  delay( 5 );
                  DialogButton( &dia->OkR, False );

               }

               clickTime = (unsigned) opEvent.eventTime;

               break;

	    } /* end of in file rect */

            if( XYInRect(x,y,&fileSBar.SBRect ) ) {
               /* scroll around */

               /* undo any old highlite selection */
               if( fR.Xmin != -1 ) 
                  InvertRect( &fR );
	       fR.Xmin = -1;

               DoScrollBar( &fileSBar );
	       numleft = fileSBar.maxval - fileSBar.curval + 1;
	       break;
            }

            /* not in any rect we care about */
            break;

         case evntKEYDN: 
            
	    /* if enter, try to OK */
            if( opEvent.eventChar == 0x0D && okenabled ) {
               retcode = Ok;
               done = True;

               /* turn OK button on a moment */
               DialogButton( &dia->OkR, True );
               delay( 5 );
	       DialogButton( &dia->OkR, False );

               break;
	    }

            /* start file display at the key */ 

            entrptr = entries;
            for( i = 0; i < numentries; i++ ) {
               if( entrptr->fileName[0] >= toupper( opEvent.eventChar ) )
                  break;
               entrptr++;
	    }
            
            /* find number of entries left */
            numleft = numentries - i;

            /* if no match, reset to start */
	    if( numleft <= 0 ) {
               entrptr = entries;
	       numleft = numentries;
               i = 0;
            }

            /* undo any old highlite selection */
            if( fR.Xmin != -1 ) 
               InvertRect( &fR );
            fR.Xmin = -1;

            DisplayFiles( numleft, entrptr );

            /* reset scroll bar position */
	    fileSBar.curval = i;
            PositionSB( &fileSBar );
            break;

         default:    /* not an event we care about */
            continue;


      }   /* end of switch */

   }   /* end of while(!done) */

   /* free memory */
   free( entries );

   /* pop dialog off */
   HideDialog(diaFile, dia ); 


   return retcode;

}



int CompressPath( char * );


void SplitSpec( char *path, char *spec )
/*
Separate any path part from spec part and add it to existing
complete 'path'.
Handles  '.' and '..' and 'drive:'
Will yield the simplest complete path posible (no . or .. )
*/

{

   char *start, *end, *p;
   


   /* if path doesn't alread end in '\', make it (like 'c:') */
   p = path + strlen( path ) - 1;
   if( *p != '\\' )
      strcat( path, "\\" );

   /* search for any indication that spec contains some path info */
   end = spec + strlen(spec);
   while( end >= spec && *end != '\\' && *end != ':' ) 
      end--;
      
   /* if we have some path info */
   if( end >= spec ) {

      /* if specing new drive, replace old path entirely */
      if( spec[1] == ':' )
         start = path;
      else
         /* if specing from root, replace all but drive in old path */
         if( spec[0] == '\\' )
	    start = path + 2;   /* skip d: */
         else
            /* add to old path */
            start = path + strlen(path);

      /* move path info from 'spec' to 'path' */
      p = spec;    
      while( p <= end ) {
         *start++ = toupper(*p);
         p++;
      }
      *start = '\0';

      /*  remove path from spec */
      strcpy( spec, end + 1 );

   }

   
   /* handle blank file spec */
   if( spec[0] == '\0' )
      strcpy( spec, "*.*" );

   /* handle . and .. file specs */
   if( spec[0] == '.' ) {

      if( spec[1] == '.' )
         strcat( path, "..\\" );

      strcpy( spec, "*.*" );
   }

   /* handle . and .. in path, compress them out */
   /* start compressions from the first '\' */
   while( CompressPath( path + 2 )  )
      ;

}



int CompressPath( char *path )
/*
Compress out a . or .. path spec. If a compression took place, returns
True, if nothing left to compress, returns false.
To completely compress out multiple occurrances of tree traversing .. 
path specs, this routine must be called until it returns False (i.e. like 
a bubble sort)
*/
{
   
   int i;
   char *p;

   p = path + 1;
   i = False;

   while( *p ) {

      if( *p == '\\' ) {

         /* sniff ahead a bit */
         if( *(p+1) == '.' ) {

            if( *(p+2) == '.' && *(p+3) == '\\' ) {
               strcpy( path, p+3 ); /* remove to last '\' */
               i = True;
            }
            else  {
               if( *(p+2) == '\\' ) {
                  strcpy( p, p+2 );     /* remove to this '\' */
                  i = True;
               }
            }
               
            /* if we compressed, will need another iteration */
            if( i )
               return True;
         }
         
	 path = p;

      }   /* end of *p = '\\' */

      p++;

   }   /* end of while *p */

   return False;
   
}


void ScrollFiles( int scrollCode )
/*
This is called via function pointer from DoScrollBar to do the
scroll bar thing for the file dialog box.
*/
{
   
   int pageworth, numleft;
   rect fR;

   /* anything to scroll ? */
   if( fileSBar.maxval == 0 )
      return;

   DupRect( &fileDlgBox.BoxR, &fR );
   InsetRect( &fR, 2, 2 );
   pageworth = (fR.Ymax - fR.Ymin) / scrnPort->txFont->lnSpacing;
   numleft = fileSBar.maxval - fileSBar.curval + 1;


   /* leave a line from last page for context */
   pageworth--;

   switch( scrollCode ) {

      case up:
         if( fileSBar.curval > fileSBar.minval )
	    fileSBar.curval--;
         break;
         
      case down:
         if( fileSBar.curval < fileSBar.maxval )
            fileSBar.curval++;
         break;

      case pgup:
         fileSBar.curval -= pageworth;
         if( fileSBar.curval < fileSBar.minval )
            fileSBar.curval = fileSBar.minval;
         break;

      case pgdown:
         if( numleft <= pageworth )
            break;
         fileSBar.curval += pageworth;
         if( fileSBar.curval > fileSBar.maxval )
            fileSBar.curval = fileSBar.maxval;
	 break;
         
      case position:
         break;

      default:
         return;
   }
   
   numleft = fileSBar.maxval - fileSBar.curval + 1;

   DisplayFiles( numleft, entries + fileSBar.curval );

}

void DisplaySpec( char *path, char *spec )
/*
Display the file path and spec in the file dialog box 
*/
{
   dialogBox *dia;
   int x, y, charwidth;
   
   dia = &fileDlgBox;
   ProtectRect(&dia->BoundRect);

   PenColor( Black );
   BackColor( White );
   RasterOp( zREPz );

   /* clear dir box */
   EraseRect( &dia->TxR );

   /* clear a filespec box */
   EraseRect( &dia->KeyR );
   FrameRect( &dia->KeyR );

   charwidth = scrnPort->txFont->chWidth;

   /* display current path */
   TextAlign( alignLeft, alignMiddle );
   ClipRect( &dia->TxR );
   PenColor( colors->text );
   y = (dia->TxR.Ymax + dia->TxR.Ymin + 1 ) / 2;
   x = dia->TxR.Xmin + charwidth;
   MoveTo( x, y );
   DrawString( path );

   /* display current spec */
   ClipRect( &dia->KeyR );
   y = (dia->KeyR.Ymax + dia->KeyR.Ymin + 1 ) / 2;
   x = dia->KeyR.Xmin + charwidth + 2;
   MoveTo( x, y );
   DrawString( spec );

   ClipRect( &(scrnPort->portRect) );
   ProtectRect(NULL);

}


void DisplayFiles( int numentry, dirRec *startentry )
/*
Display as many entries as possible up to numentry, in the file dialogs 
BoxR. Start displaying them from startentry in the entries array.
Leaves global entrptr pointing to the entries element which will be at
the top of the box (startentry).
*/
{
   dialogBox *dia;
   int i, x, y, pageworth, charwidth, charheight, fast;
   char tmp[MAXLEN], *p;
   rect fR;
	 fontRcd *userfnt;

   /* check for special case of scroll one line up or down */
   /* use fast scrollrect in those cases, else repaint all */
   fast = 0;
   if( startentry == entrptr - 1 )
      fast = 1;
   if( startentry == entrptr + 1 )
      fast = 2;

   /* set global entries pointer to new top of entries */
   entrptr = startentry;

   dia = &fileDlgBox;
   ProtectRect(&dia->BoundRect);
   DupRect( &dia->BoxR, &fR );
   InsetRect( &fR, 2, 2 );


    /* switch to system font while drawing view box */
   userfnt = scrnPort->txFont;
   SetFont( userfnt );

   charwidth = scrnPort->txFont->chWidth;
   charheight = scrnPort->txFont->lnSpacing;
   pageworth = (fR.Ymax - fR.Ymin) / charheight;
   BackColor( White );
   RasterOp( zREPz );

   if( fast == 0 ) {
      /* clear file box */
      PenColor( Black );
      EraseRect( &fR );
      FrameRect( &dia->BoxR );
   }

   ClipRect( &fR );

   x = fR.Xmin + charwidth;
   y = fR.Ymin + charheight;
   MoveTo( x,y );
   PenColor( colors->text );
   TextAlign( alignLeft, alignBaseline );

   if( numentry == 0 )  {
      DrawString("No Files!!");
      ClipRect( &(scrnPort->portRect) );
      ProtectRect(NULL);
      return;
   }

   if( startentry == NULL )  {
      DrawString("Too Many Files!!");
      ClipRect( &(scrnPort->portRect) );
      ProtectRect(NULL);
      return;
   }

      /* do special optimizations */
   switch( fast ) {
      
      case 1:
         /* scroll down one */
         i = ( pageworth - 1 ) * charheight;
         fR.Ymax = fR.Ymin + i + scrnPort->txFont->descent;
         ScrollRect( &fR, 0, charheight );
         /* make it draw only one new line */
         pageworth = 1;
         break;

      case 2:
         /* scroll up one */
         i = charheight + scrnPort->txFont->descent;
         fR.Ymin += i;
         ScrollRect( &fR, 0, -charheight );
         if( numentry >= pageworth ) {
            /* redraw the new line at the bottom */
            startentry += pageworth - 1;
            y += charheight * ( pageworth - 1 );
            pageworth = 1;
         }
         else
            pageworth = 0;  /* don't redraw anything */

         break;
   
   }   /* end of switch( fast ) */


   /* if less than a page to display, only display that much */
   if( numentry < pageworth )
      pageworth = numentry;

   /* draw pageworth lines in rect fR */
   x = fR.Xmin + charwidth;
   /* byte align for speed */
   x += x % 8;

   for( i = 0; i < pageworth; i++ ) {

      MoveTo( x, y);

      /* is it a sub dir name ? */
      if( startentry->fileAttr & 0x10 )
         DrawChar( '\\' );

      DrawString( startentry->fileName );

      /* next line */
      startentry++;
      y += charheight;


   }   /* end of for pageworth */
    
   ClipRect( &(scrnPort->portRect) );
   ProtectRect(NULL);

}


int SortDir( const void *dir1, const void *dir2 )
/*
Sort two dirRec entries by file name then extension.
Intended to be called by qsort
*/
{
   char *p1, *p2;
   
   p1 = ( (dirRec *) dir1)->fileName;  p2 = ( (dirRec *) dir2)->fileName;

   /* first compare file name */
   while( *p1 && *p1 != '.' && *p2 && *p2 != '.' ) {

      if( *p1 > *p2 )
         return 1;
      if( *p1 < *p2 )
         return -1;

      p1++; p2++;
   }

   /* then compare extension */
   while( *p1 && *p2 ) {

      if( *p1 > *p2 )
         return 1;
      if( *p1 < *p2 )
         return -1;

      p1++; p2++;
   }

   /* longer is bigger */
   if( *p1  )
      return 1;
   if( *p2  )
      return -1;

   return 0;
}

int SelectFile( int x, int y, rect *fileR, int num )
/*
Figure out which file is selected by the mouse, return an index from
0 to (num - 1). This index may then be used to address the selected item.
Returns -1 if no valid item selected.

x,y     = the point selected
fileR   = a rectangle describing the last inverted selection, or Xmin = -1
        if no currently inverted selection
num     = the number of total items that could potentialy be displayed
*/

{
   
   int i, charheight, pageworth;
   rect tR;

   DupRect( &fileDlgBox.BoxR, &tR );

   ProtectRect( &tR );

   /* invert the old selection */
   if( fileR->Xmin != -1 ) 
      InvertRect( fileR );

   /* find the new selection */
   InsetRect( &tR, 2, 2 );
   charheight = scrnPort->txFont->lnSpacing;
   pageworth = (tR.Ymax - tR.Ymin) / charheight;

   /* make a small rect one line deep */
   tR.Ymax = tR.Ymin + charheight;
   OffsetRect( &tR, 0, scrnPort->txFont->descent );

   /* search for the line selected */
   if( num < pageworth )
      pageworth = num;

   for( i = 0; i < pageworth; i++ ) {
      if( XYInRect( x, y, &tR ) )
         break;
      OffsetRect( &tR, 0, charheight );
   }
   if( i >= pageworth )  {
      /* outside of selectable items */
      fileR->Xmin = -1;
      ProtectRect(NULL);
      return -1;
   }

   /* save this as last item inverted */
   DupRect( &tR, fileR );

   InvertRect( fileR );

   ProtectRect(NULL);

   return i;

}

