//--------------------------------------------------------//
//                                                        //
//   File:  MWBMP.CPP  all bmp code                       //
//                                                        //
//   Desc:    Class and structure definitions for the     //
//            Windows BMP format - DOS version            //
//                                                        //
//--------------------------------------------------------//

#include "stdlib.h"
#include "stdio.h"
#include "string.h"

#include "bmp.hpp"
#include "metawndo.h"      /* master MetaWINDOW include file */
#include "GUIconst.h"
#include "GUIextrn.h"
extern int meta_hd_size;
extern dialogBox mainDlgBox;
extern int *clrmap;
extern palData   PCXColors[256];
static   palData   RGBdata[256];
void Pal_Table(void);
int bmp_width,bmp_height;

int load_bmp( char *fn,rect *wR );//load BMP file
int BMP_Save(char *filename,rect *R);//Save the BMP file
int BMPmain( void);//file dialog for BMP files

 int BMP_Save(char *filename,rect *R)
//.......Save the BMP file
{
    int i,j;

    rgb pal[256];
    image *pkPtr,*scanln,*buf;
    rect sR;
   grafPort *wPort;
   GetPort(&wPort);
   DupRect( R, &sR );
   meta_hd_size=(sizeof(imageHeader));

   int nrows   = sR.Ymax-sR.Ymin;
   int ncols   = sR.Xmax-sR.Xmin;
  //int ncolors = QueryColors() +1;
    int nbits=wPort->portMap->pixBits*wPort->portMap->pixPlanes;
   int ncolors  = (nbits <= 8) ? 1 << nbits : 0;
   int rowbytes = ((ncols*nbits + 31) / 32) * 4;
   int maxcolors=QueryColors()+1;

   //use 5*rowbytes for XlateImage if image is 4 planes

   scanln = (image *)malloc((unsigned int)rowbytes*5);
   if( scanln == NULL ) {
   Dialog( diaAlert, "Not enough memory!", "for scanline" );
   return 0;
   }
   buf = (image *)malloc((unsigned int)rowbytes*5);

   if( buf == NULL ) {
	free(scanln);
	Dialog( diaAlert, "Not enough memory!", "for buf" );
	return  0;
   }


   FILE *f = fopen( filename, "wb" );
   if( f == 0 )
   {
	Dialog( diaAlert, "File Write Error!", "Try Again" );
	free(scanln);
	free(buf);
	return  0;
   }


   // write an empty file header as a place holder
   xBITMAPFILEHEADER fh;
   if( fwrite( &fh, sizeof(xBITMAPFILEHEADER), 1, f ) != 1 )
	{
	Dialog( diaAlert, "File Write Error!", "Try Again" );
      fclose(f);
	free(scanln);
	free(buf);

	return  0;
   }


   // initialize and write the image header
   xBITMAPINFOHEADER ih;
   ih.biSize = sizeof(xBITMAPINFOHEADER);
   ih.biWidth = ncols;
   ih.biHeight = nrows;
   ih.biPlanes = 1;
   ih.biBitCount = nbits;
   ih.biSizeImage = ((long) nrows)  * ((long) rowbytes);
   if( fwrite( &ih, sizeof(xBITMAPINFOHEADER), 1, f ) != 1 )
	{
	Dialog( diaAlert, "File Write Error!", "Try Again" );
	 fclose(f);
	free(scanln);
	free(buf);

	return  0;
	}


   // write the palette
   xRGBQUAD x;

     ReadPalette( 0, 0,QueryColors() , RGBdata );

	for( i=0; i<ncolors; i++ )
	{

	  RGBdata[i].palRed /=256;
	  RGBdata[i].palGreen /= 256;
	  RGBdata[i].palBlue /= 256;
	pal[i].red=(unsigned char)RGBdata[i].palRed;
	pal[i].grn=(unsigned char)RGBdata[i].palGreen;
	pal[i].blu=(unsigned char)RGBdata[i].palBlue;

	 x.set( pal[i].red, pal[i].grn, pal[i].blu );
	 if( fwrite( &x, sizeof(xRGBQUAD), 1, f ) != 1 )
	   {
	Dialog( diaAlert, "#BMP File Error!",NULL );
	 fclose(f);
	free(scanln);
	free(buf);

	    return 0;
	   }
      }


   // save offset to start of bitmap
   long bmofs = ftell( f );


   // output the bitmap
   BmpEncoder bmp( f, nbits );

    sR.Ymin =sR.Ymax- 1;   //bmp file is upside down

   if(maxcolors>16){       //256 color mode
   for( i=nrows-1; i>=0; i-- )
   {

     ReadImage(&sR,scanln);
     sR.Ymin -= 1;
     sR.Ymax -= 1;

	for(j=0;j<ncols;j++)
	buf[j]=scanln[j+meta_hd_size];  //minus imageHeader (12 bytes) for ReadImage buf record

     bmp.encode( buf, ncols);
	if( bmp.status() != xOKAY )
      {
	 Dialog( diaAlert, "Error with File", "BMP Error" );
	 free(scanln);
	 free(buf);
	 fclose( f );
	 return 0;
	}
   }
  } //end of if maxcolors >16

  if(maxcolors<=16){      // 16 color mode

     if( ( pkPtr = (image *)( malloc( (unsigned int) rowbytes*5 ) ) )  == NULL )
	{  fclose( f );
	   free(scanln);
	    free(buf);
	    return 0 ;
	}
    for( i=nrows-1; i>=0; i-- )
	{

	ReadImage(&sR, pkPtr);
	 sR.Ymin -= 1;
	 sR.Ymax -= 1;

	XlateImage(pkPtr,buf,8,1,NULL);   //all images are stored in 8 bit images
	 for(j=0;j<ncols;j++)
	 scanln[j]=buf[j+meta_hd_size];  //add imageHeader (12 bytes) for ReadImage buf record

	   bmp.encode( scanln, ncols);
	if( bmp.status() != xOKAY )
      {

	  Dialog( diaAlert, "Error with File", "encoder error" );

	 free(pkPtr);
	 free(scanln);
	 free(buf);
	 fclose( f );
	 return 0;
	}
     }
  } //end of 16 color mode

   if(maxcolors<=16)free(pkPtr);
   free(scanln);
   free(buf);

   // save offset to end of file
   long fisiz = ftell( f );

   // update and rewrite file header
   fh.bfSize = fisiz;
   fh.bfOffBits = bmofs;
   fseek( f, 0L, SEEK_SET );
   if( fwrite( &fh, sizeof(xBITMAPFILEHEADER), 1, f ) != 1 )
	{
	Dialog( diaAlert, "File Write Error!", "Try Again" );
	return  0;
   }


   fclose( f );
   return 1;
}


int load_bmp( char *fn,rect *wR )
{

    imageHeader    *imPtr;
    image *buf2,*pkPtr,*buf,*scanln;
    int j,i,colortable=0,err;
    rect box,sR;
    rgb  *imgpal;
     int width, height, depth, rle,ncolors,rwbytes;
     grafPort *wPort;


    meta_hd_size=(sizeof(imageHeader));

    if ( wR != NULL )  DupRect( wR, &box );

   // open file
   FILE *fbmp = fopen( fn, "rb" );
   if( fbmp == 0 )
	{
	Dialog( diaAlert, "File Read Error!", "Try Again" );
	return 0;
   }


   // read header and check
   xBITMAPFILEHEADER bmfh;
   if( fread(&bmfh,sizeof(xBITMAPFILEHEADER),1,fbmp) != 1 )
	  {
	Dialog( diaAlert, "File Read Error!", "Try Again" );
	fclose(fbmp);
	return 0;
   }

   if( bmfh.bfType != BMP_SIGNATURE_WORD )
	 {
	Dialog( diaAlert, "File Read Error!", "Try Again" );
	fclose(fbmp);
	return 0;
   }


   // determine which format version is used
   // from structure size field
   long offset = ftell( fbmp );
   long nbytes = 0;
   fread( &nbytes, sizeof(long), 1, fbmp );
   fseek( fbmp, offset, SEEK_SET );

   // read bitmap header and palette

   // pre Windows 3.0 version
   if( nbytes == sizeof(xBITMAPCOREHEADER) )
   {
	xBITMAPCOREHEADER bmch;
	if( fread(&bmch,sizeof(xBITMAPCOREHEADER),1,fbmp) != 1 ){
	 fclose(fbmp);
	return 0;
	}
	width   = bmch.bcWidth;
	height  = bmch.bcHeight;
	depth   = bmch.bcBitCount;
	rle     = 0;
	ncolors = 1 << depth;
	rwbytes=depth*width;

	if( ncolors > 0 )
	{
	 imgpal = new rgb [ncolors];
	   if( imgpal == 0 )
	    return 0;
	 xRGBTRIPLE t;
	 for(  i=0; i<ncolors; i++ )
	   {
	    if( fread(&t,sizeof(xRGBTRIPLE),1,fbmp) != 1 )
		     {
	Dialog( diaAlert, "File Read Error!", "Try Again" );
	fclose(fbmp);
	return 0;
   }

	    imgpal[i] = rgb(t.rgbRed,t.rgbGreen,t.rgbBlue);

	RGBdata[i].palRed=imgpal[i].red;
	RGBdata[i].palGreen=imgpal[i].grn;
      RGBdata[i].palBlue= imgpal[i].blu;
	RGBdata[i].palRed   *= 256;
	RGBdata[i].palGreen *= 256;
	RGBdata[i].palBlue  *= 256;

	 }
	}

   }

   // Windows 3.0 and later version
   else if( nbytes == sizeof(xBITMAPINFOHEADER) )
   {
	xBITMAPINFOHEADER bmih;
      if( fread(&bmih,sizeof(xBITMAPINFOHEADER),1,fbmp) != 1 )
	{
	Dialog( diaAlert, "File Read Error!", "Try Again" );
	fclose(fbmp);
	return 0;
	}

      width   = (int) bmih.biWidth;
	height  = (int) bmih.biHeight;
	depth   = bmih.biBitCount;
	rle     = (int) bmih.biCompression;
	ncolors = 2;
	//ncolors = 1 << depth;   //did not seem to work all the time
	rwbytes=depth*width;

     if(depth>2)ncolors=16;
     if(depth>4)ncolors=256;   //8 bits
     if(depth>8)ncolors=0;      //24 bits


	if( ncolors > 0 )
      {
	   imgpal = new rgb [ncolors];
	  if( imgpal == 0 )
	  {
	  Dialog( diaAlert, "File Read Error!", "Try Again" );
	  fclose(fbmp);
	  return 0;
	  }

	   xRGBQUAD q;
    for( i=0; i<ncolors; i++ )
    {

	if( fread(&q,sizeof(xRGBQUAD),1,fbmp) != 1 ) {
	Dialog( diaAlert, "File Read Error!", "Try Again" );
	fclose(fbmp);
	return 0;
	}

	imgpal[i] = rgb(q.rgbRed,q.rgbGreen,q.rgbBlue);
	RGBdata[i].palRed=imgpal[i].red;
	RGBdata[i].palGreen=imgpal[i].grn;
      RGBdata[i].palBlue= imgpal[i].blu;
	RGBdata[i].palRed   *= 256;
	RGBdata[i].palGreen *= 256;
	RGBdata[i].palBlue  *= 256;
     } //end of for loop
    }//end of ncolors>0
  }//end of else if

   // unknown format
   else
   {
	Dialog( diaAlert, "File Read Error!", "Format error or input file not a BMP file" ) ;
	fclose(fbmp);
	return 0;
   }

   // position to start of bitmap
   fseek( fbmp, bmfh.bfOffBits, SEEK_SET );
   if(ncolors>0){
   //use this for color mapping
   for( i=0; i<256; i++ )
   {
	    PCXColors[i].palRed = RGBdata[i].palRed;
	    PCXColors[i].palGreen = RGBdata[i].palGreen;
	    PCXColors[i].palBlue = RGBdata[i].palBlue;
   }
   WritePalette( 0, 0,QueryColors() , RGBdata );
   }
     PenColor(0);
     GetPort(&wPort);

	if(wR==NULL)FillRect(&wPort->portRect,1);
	 else FillRect(&box,1);


     scanln = (image *)malloc((unsigned int)width*5);
	if( scanln == NULL ) {
	Dialog( diaAlert, "Not enough memory!", "for scanline" );
	fclose(fbmp);
	return 0;
	}

	buf = (image *)malloc((unsigned int)width*5);
	if( buf == NULL ) {
	fclose(fbmp);
	free(scanln);
	Dialog( diaAlert, "Not enough memory!", "for buf" );
	return 0;
	}
	   buf2 = (image *)malloc((unsigned int)width*5);

   if( ( pkPtr = (Byte *)( malloc( (unsigned int) rwbytes ) ) )  == NULL ){    free(pkPtr); free(buf2);
	fclose(fbmp);
	free(scanln);
	free(buf);
	free(buf2);
	return 0;
	}



    SetRect(&sR,0,0,width,1);
    ReadImage(&sR,buf);   //neeed ImageHearder Info
     // ReadImage(&sR,pkPtr);

    if(wR!=NULL){  //used to display image in box rect
   SetRect(&sR,box.Xmin,box.Ymin,box.Xmin+width,box.Ymin+1);
   sR.Ymax =sR.Ymin+height;
   sR.Ymin =sR.Ymax-1;
     }

     else{
     sR.Ymax =sR.Ymin+height+10; //add 10 to display lower on screen
     sR.Ymin =sR.Ymax-1;
	}


   BmpDecoder bmp( fbmp, rle, depth );
   bmp.init( );

   int  maxcolors=QueryColors()+1;
  if((maxcolors>16) ||  ((ncolors==0)&&(wPort->portMap->pixBits == 24)) )
 {  // 256 color   or 24 bit?

	for( i=height; i>0; i-- )
	{
	bmp.decode( scanln, width );
	for(j=0;j<width;j++)
	 buf[j+meta_hd_size]=scanln[j];     //add imageHeader (12 bytes) for WriteImage buf record

	WriteImage(&sR,buf);
	 sR.Ymin -=1;
	 sR.Ymax -=1;

	 }
    } //end of 256 color

   if(maxcolors<=16){   //16 color mode

	if(ncolors>16) {
	     //ShowCursor();
	     //err= Dialog( diaNull, "Convert 256 Color Graphic file ","to current video mode" );
	     //HideCursor();
	//if(err!=0)
	Pal_Table();colortable=1; //color table mapping
	 }
   // ReadImage(&sR,pkPtr);

   imPtr = (imageHeader *)pkPtr;       // pointer to MW image header
   imPtr->imWidth    = width;
   imPtr->imHeight   = 1;
   imPtr->imAlign    = 0;
   imPtr->imBits=8;
   imPtr->imPlanes=1;
   imPtr->imRowBytes=rwbytes; //can be 24 bits per pixel.. 3 planes?


   for( i=height; i>0; i-- )
   {
	bmp.decode( buf, width );
    if(colortable==1){        //color mapping
	  for( j=0; j<width; j++ )
	  buf2[j] = clrmap[buf[j]];
	  for(j=0;j<width;j++)
	  pkPtr[j+meta_hd_size]=buf2[j];     //add imageHeader (12 bytes) for WriteImage buf record

     }

     else{     //no color mapping
	for(j=0;j<width;j++)
	 pkPtr[j+meta_hd_size]=buf[j];     //add imageHeader (12 bytes) for WriteImage buf record
	}
	memset(buf2,0x00,sizeof(buf2));
	XlateImage(pkPtr,buf2,1,4,NULL);
	WriteImage(&sR,buf2);


	 sR.Ymin -=1;
	 sR.Ymax -=1;

     }
   }//end of 16 color mode
    bmp.term();
    fclose( fbmp );

    free(buf2);
    free(pkPtr);
    if(colortable==1) free(clrmap);
    free(scanln);
    free(buf);
    if(imgpal)delete imgpal;
    ShowCursor();   //had problems with Cursor not showing up again

   return 1;
}

int BMPmain( void)
{
	char     xfile[80];
	rect wR;
	int err;
	strcpy( xfile,"*.bmp" );
	if ( Dialog( diaFile, "BMP files", xfile ) == 0 )
		return 0;

		 char *p = strchr( xfile, '.' );
	 if( p == 0 )
	    strcat( xfile, ".bmp" );
     HideCursor();DupRect( &scrnPort->portRect, &mainDlgBox.BoundRect );

   GUIWindow( &mainDlgBox, xfile );

  err= load_bmp(xfile,NULL);
   if(err==0){

   ShowCursor();
   return 0;
   }
   ShowCursor();
   return 1;
}


//..................struct xBITMAPFILEHEADER

void xBITMAPFILEHEADER::list( void )
{
   printf( "xBITMAPFILEHEADER::bfType      = '%c%c'\n",
		bfType & 0xFF, (bfType>>8) & 0xFF );
   printf( "xBITMAPFILEHEADER::bfSize      = %lu\n",
		bfSize );
   printf( "xBITMAPFILEHEADER::bfReserved1 = %d\n",
		bfReserved1 );
   printf( "xBITMAPFILEHEADER::bfReserved2 = %d\n",
            bfReserved2 );
   printf( "xBITMAPFILEHEADER::bfOffBits   = %lu\n",
            bfOffBits );
}

//..................struct xRGBTRIPLE

void xRGBTRIPLE::list( int index )
{
   printf( "xRGBTRIPLE[%3d]::  ", index );
   printf( "rgbRed=%3d  rgbGreen=%3d  rgbBlue=%3d\n",
            rgbRed&0xFF, rgbGreen&0xFF, rgbBlue&0xFF );
}

//..................struct xBITMAPCOREHEADER

void xBITMAPCOREHEADER::list( void )
{
   printf( "xBITMAPCOREHEADER::bcSize     = %lu\n",
            bcSize );
   printf( "xBITMAPCOREHEADER::bcWidth    = %d\n",
            bcWidth );
   printf( "xBITMAPCOREHEADER::bcHeight   = %d\n",
            bcHeight );
   printf( "xBITMAPCOREHEADER::bcPlanes   = %u\n",
		bcPlanes );
   printf( "xBITMAPCOREHEADER::bcBitCount = %u\n",
            bcBitCount );
}

//..................struct xRGBQUAD

void xRGBQUAD::list( int index )
{
   printf( "xRGBQUAD[%3d]::  ", index );
   printf( "rgbRed=%3d  rgbGreen=%3d  rgbBlue=%3d",
            rgbRed&0xFF, rgbGreen&0xFF, rgbBlue&0xFF );
   printf( "  rgbReserved=%d\n", rgbReserved );
}

//..................struct xBITMAPINFOHEADER

void xBITMAPINFOHEADER::list( void )
{
   printf( "xBITMAPINFOHEADER::biSize          = %lu\n",
		biSize );
   printf( "xBITMAPINFOHEADER::biWidth         = %ld\n",
            biWidth );
   printf( "xBITMAPINFOHEADER::biHeight        = %ld\n",
            biHeight );
   printf( "xBITMAPINFOHEADER::biPlanes        = %d\n",
            biPlanes );
   printf( "xBITMAPINFOHEADER::biBitCount      = %d\n",
		biBitCount );
   printf( "xBITMAPINFOHEADER::biCompression   = %lu\n",
            biCompression );
   printf( "xBITMAPINFOHEADER::biSizeImage     = %lu\n",
            biSizeImage );
   printf( "xBITMAPINFOHEADER::biXPelsPerMeter = %ld\n",
		biXPelsPerMeter );
   printf( "xBITMAPINFOHEADER::biYPelsPerMeter = %ld\n",
            biYPelsPerMeter );
   printf( "xBITMAPINFOHEADER::biClrUsed       = %lu\n",
            biClrUsed );
   printf( "xBITMAPINFOHEADER::biClrImportant  = %lu\n",
		biClrImportant );
}

//..................class BmpDecoder

BmpDecoder::BmpDecoder( FILE *f_in, int rle, int nbits ) :
            Decoder( tRLE )
{
   cond = xOKAY;
   rle_method = rle;
   pxl_size = nbits;
   bmp_file = f_in;
}

int BmpDecoder::status( void )
{
   if( ferror(bmp_file) ) cond = xIOERROR;
   if( feof(bmp_file) ) cond = xOUTOFSYNC;
   return cond;
}

int BmpDecoder::init( void )
{
   return status();
}

int BmpDecoder::term( void )
{
   return status();
}

void BmpDecoder::read_none( unsigned char *buf, int npxls )
{
   int i, j, nbytes;
   switch( pxl_size )
   {
      case 1 :
         nbytes = (npxls + 7) / 8;
         nbytes = (nbytes + 3) / 4;
	   nbytes *= 4;
	   while( npxls > 0 )
         {
            nbytes--;
            int x = fgetc( bmp_file );
            int m = 0x80;
		while( (npxls > 0) && (m != 0) )
		{
               *buf++ = (x & m) ? 1 : 0;
		   m >>= 1;
               npxls--;
            }
         }
	   while( nbytes-- > 0 )
		fgetc( bmp_file );
         break;

      case 4 :
         nbytes = (npxls + 1) / 2;
         nbytes = (nbytes + 3) / 4;
	   nbytes *= 4;
	   while( npxls > 0 )
         {
            nbytes--;
            int x = fgetc( bmp_file );
		int p2 = x & 0x0F;
            int p1 = (x>>4) & 0x0F;
            *buf++ = p1;
            npxls--;
		*buf++ = p2;
            npxls--;
         }
	   while( nbytes-- > 0 )
		fgetc( bmp_file );
	   break;

      case 8 :
         nbytes = (npxls + 3) / 4;
         nbytes *= 4;
         fread( buf, npxls, 1, bmp_file );
	   nbytes -= npxls;
	   while( nbytes-- > 0 )
            fgetc( bmp_file );
         break;

      case 24 :
	   nbytes = (npxls*3 + 3) / 4;
         nbytes *= 4;
         fread( buf, npxls, 3, bmp_file );
	   nbytes -= npxls*3;
         while( nbytes-- > 0 )
		fgetc( bmp_file );
         // reorder bgr to rgb
         for( i=0, j=0; i<npxls; i++, j+=3 )
	   {
            int k = buf[j];
            buf[j] = buf[j+2];
            buf[j+2] = k;
         }
	   break;
   }
}

void BmpDecoder::read_rle4( unsigned char *buf, int npxls )
{
   int b1, b2;
   while( cond == xOKAY )
   {
      b1 = fgetc( bmp_file );
	b2 = fgetc( bmp_file );
	if( b1 == 0 )  //.... absolute or escape
      {
	   switch( b2 )
         {
		case 0 :  //.... end-of-line
               return;

            case 1 :  //.... end-of-image
               cond = xENDOFIMAGE;
               return;

		case 2 :  //.... delta
		   cond = xUNSUPPORTED;
               return;

		default : //.... nonrepeating run
               while( b2-- )
               {
                  if( npxls > 0 )
			{
                     *buf++ = fgetc( bmp_file );
                     npxls--;
                  }
               }
		   break;
         }
      }
      else  //............. encoded
	{
         unsigned char p1 = b2      & 0x0F;
	   unsigned char p2 = (b2>>4) & 0x0F;
	   unsigned char p;
         if( b1 & 1 ) { p=p1; p1=p2; p2=p; }
         while( b1-- )
         {
            if( npxls > 0 )
            {
               *buf++ = (b1 & 1 ) ? p1 : p2;
		   npxls--;
		}
         }
	}
   }
}

void BmpDecoder::read_rle8( unsigned char *buf, int npxls )
{
   int b1, b2;
   while( cond == xOKAY )
   {
	b1 = fgetc( bmp_file );
	b2 = fgetc( bmp_file );
      if( b1 == 0 )  //.... absolute or escape
      {
	   switch( b2 )
         {
            case 0 :  //.... end-of-line
		   return;

		case 1 :  //.... end-of-image
               cond = xENDOFIMAGE;
               return;

            case 2 :  //.... delta
		   cond = xUNSUPPORTED;
               return;

		default : //.... nonrepeating run
               while( b2-- )
		   {
			if( npxls > 0 )
			{
                     *buf++ = fgetc( bmp_file );
                     npxls--;
                  }
               }
		   break;
         }
      }
	else  //............. encoded
	{
         while( b1-- )
         {
            if( npxls > 0 )
		{
               *buf++ = b2;
               npxls--;
            }
	   }
      }
   }
}

int BmpDecoder::decode( unsigned char *buf, int npxls )
{
   switch( rle_method )
   {
      case xBI_NONE :
           read_none( buf, npxls );
	     break;

      case xBI_RLE8 :
           read_rle8( buf, npxls );
           break;

      case xBI_RLE4 :
	     read_rle4( buf, npxls );
	     break;

      default :
	     cond = xBADPARAM;
	     break;
   }

   return status();
}

//..................class BmpEncoder

BmpEncoder::BmpEncoder( FILE *f_out, int nbits ) :
            Encoder( tRLE )
{
   pxl_size = nbits;
   bmp_file = f_out;
}

int BmpEncoder::status( void )
{
   return ferror(bmp_file);
}

int BmpEncoder::init( void )
{
   return status();
}

int BmpEncoder::term( void )
{
   return status();
}

void BmpEncoder::wrt_line( unsigned char *buf, int npxls )
{
   int i, j, nbytes, x, m;
   switch( pxl_size )
   {
      case 1 :
         nbytes = (npxls + 7) / 8;
         nbytes = (nbytes + 3) / 4;
         nbytes *= 4;
	   x = 0;
	   m = 0x80;
	   while( npxls-- > 0 )
         {
            if( *buf++ )
               x |= m;
            if( (m>>=1) == 0 )
            {
		   fputc( x, bmp_file );
		   x = 0;
               m = 0x80;
               nbytes--;
            }
         }
	   if( m != 0x80 )
         {
            fputc( x, bmp_file );
            nbytes--;
         }
         while( nbytes-- > 0 )
		fputc( 0, bmp_file );
	   break;

      case 4 :
         nbytes = (npxls + 1) / 2;
         nbytes = (nbytes + 3) / 4;
         nbytes *= 4;
         x = 0;
         while( npxls > 0 )
	   {
            x = (*buf++) << 4;
            if( npxls > 1 )
               x |= *buf++;
		fputc( x, bmp_file );
		npxls -= 2;
            nbytes--;
         }
         while( nbytes-- > 0 )
            fputc( 0, bmp_file );
         break;

	case 8 :
         nbytes = (npxls + 3) / 4;
         nbytes *= 4;
         fwrite( buf, npxls, 1, bmp_file );
         nbytes -= npxls;
         while( nbytes-- > 0 )
		fputc( 0, bmp_file );
         break;

      case 24 :
         // reorder rgb to bgr
	   for( i=0, j=0; i<npxls; i++, j+=3 )
         {
		int k = buf[j];
            buf[j] = buf[j+2];
            buf[j+2] = k;
         }
         nbytes = (npxls*3 + 3) / 4;
	   nbytes *= 4;
	   fwrite( buf, npxls, 3, bmp_file );
	   nbytes -= npxls*3;
         while( nbytes-- > 0 )
            fputc( 0, bmp_file );
         break;
   }
}

int BmpEncoder::encode( unsigned char *buf, int npxls )
{
   wrt_line( buf, npxls );
   return status();
}

