#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>
#include "metawndo.h"      /* master MetaWINDOW include file */
#include "GUIconst.h"
#include "GUIextrn.h"

#define  PACKED   0xC0  /* two hi bits of byte */
#define  MASK     0x3F  /* lower six bits */
#define  PACK      0xC0
palData   PCXColors[256];
void Pal_Table(void);  //color mapping
extern int *clrmap;
/*
This function reads the PCX header record and palette information
for a specified PCX file.
*/

int SEQ ReadPCXInfo( char *fname, pcxHead *pcxHdr, palData RGBColors[] )
{
   int       i, j;
   long int  fpos;
   FILE     *datfile;


   /* Open source file */
   if( ( datfile = fopen( fname,"r+b") ) == NULL ) /* binary mode read */
	return 0;

   /*  Read Header */
   fread( pcxHdr, sizeof(pcxHead), 1, datfile);

   /* Validate PCX file by checking */
	/* manufacturers ID (Zsoft-10) and PCX encoding (1) */
   if( (pcxHdr->pcxManf != 10) || (pcxHdr->pcxEncode != 1) )
	{
		fclose( datfile );
	   return 0;
   }

   /* convert PCX digital palette data into MetaWINDOW compatible format */
   for( i = 0; i < 16; i++ )
   {
      j = i*3;
      RGBColors[i].palRed   = (int) pcxHdr->pcxColor[j];
      RGBColors[i].palGreen = (int) pcxHdr->pcxColor[j+1];
	RGBColors[i].palBlue  = (int) pcxHdr->pcxColor[j+2];

	/* scale from 8 to 16 bit values */
      RGBColors[i].palRed   *= 256;
	RGBColors[i].palGreen *= 256;
      RGBColors[i].palBlue  *= 256;
   }

	/* check for optional 256 color palette */
   fseek( datfile, 0L, SEEK_END );   /* seek to 769 bytes from end of file */
   fpos = ftell( datfile );
   fseek( datfile, fpos - 769L, SEEK_SET ); 

   /* should be a 12 there */
   i = getc( datfile );
   if( i != 12 )
	{
      fclose( datfile );
      return 1;           /* analog palette data is optional */
   }

   /* convert PCX analog palette data into WritePalette compatible format */
   for( i=0; i<256; i++ )
   {
      RGBColors[i].palRed   = getc(datfile);
	RGBColors[i].palGreen = getc(datfile);
	RGBColors[i].palBlue  = getc(datfile);

      /* scale from 8 to 16 bit values */
      RGBColors[i].palRed   *= 256;
      RGBColors[i].palGreen *= 256;
      RGBColors[i].palBlue  *= 256;
	}
	 for( i=0; i<256; i++ )
   {
	    PCXColors[i].palRed = RGBColors[i].palRed;
	    PCXColors[i].palGreen = RGBColors[i].palGreen;
	    PCXColors[i].palBlue = RGBColors[i].palBlue;
   }
   fclose( datfile );
   return 1;
}


/*
This function copies the picture image in the specified PCX file to
the current port and bitmap. Note that using "sourceR" to read in
a specified rectangular area has not been implemented yet and will
result in a compiler warning.
*/

int SEQ ReadPCX( char *fname, rect *sourceR, rect *destR )
{
   register    Byte  unwork, unwork2;
   Byte        *unpkPtr,*wrkPtr;
   Word        x,y,z,i,lineCnt,height;
   FILE        *datfile;
   rect        workR, R;
   grafPort    *currPort, pcxPort;
   grafMap     *currBitmap;
   imageHeader *imPtr;   
   pcxHead     pcxInfo;
   int xlate=0,colortable=0,j,err;
   image   *buf2,*pcxPtr;

   GetPort(&currPort);
   currBitmap = currPort->portMap;

   /* Open source file */
   if( ( datfile = fopen( fname,"r+b") ) == NULL ) /* binary mode read */
	return 0;

   /* Read Header Record */
   fread( &pcxInfo, sizeof(pcxInfo), 1, datfile);

   if( (pcxInfo.pcxBits != (Byte)currBitmap->pixBits) ||
				     (pcxInfo.pcxPlanes != (Byte)currBitmap->pixPlanes) )
   {
	 //ShowCursor();

     //err=	 Dialog( diaNull, "Convert Graphic file ","to current video mode" );
    //HideCursor();

    //if(err==0){ fclose( datfile );return 0;}

	xlate=1;
   }
   if( (pcxInfo.pcxBits != (Byte)currBitmap->pixBits) &&
	 (pcxInfo.pcxPlanes ==1 ) &&
	 ((Byte)currBitmap->pixPlanes==4) )

   {
	  ShowCursor();

	err=	 Dialog( diaNull, "Convert 256 Color Graphic file ","to current video mode" );
    HideCursor();
    if(err!=0){
	Pal_Table();
	colortable=1;
     }
   }


   lineCnt = pcxInfo.pcxRowBytes * pcxInfo.pcxPlanes;  

   if( ( unpkPtr = (Byte *)(malloc( lineCnt + sizeof(imageHeader) ) ) ) == NULL  )
   {
	fclose( datfile );
	return 0;

   }

   /* Fill in header for WriteImage */
   imPtr = (imageHeader *) unpkPtr;

   /* PCX info considers Xmax to be Xmax-1 (ie 639 for 640x480) */
   imPtr->imWidth    = (pcxInfo.pcxWndo.Xmax+1) - pcxInfo.pcxWndo.Xmin;
   imPtr->imHeight   = 1;
   imPtr->imAlign    = 0;
   imPtr->imRowBytes = pcxInfo.pcxRowBytes;
   imPtr->imBits     = pcxInfo.pcxBits;
   imPtr->imPlanes   = pcxInfo.pcxPlanes;



   /* check if the destination rect pointer is null */
   if ( destR == NULL )
      DupRect( &(currPort->portRect), &workR );  /* use current port's rect */
   else
      DupRect( destR, &workR );

   /* must do all work in global coordinates */
   if ( currPort->portFlags & pfVirtual ) /* using virtual coordinates? */
	Vir2GblRect( &workR );        /* translate rect to read file */
   else /* in case we are using local coordinates */
	Lcl2GblRect( &workR );

   InitPort( &pcxPort );     /* use a fresh port with everything in global */
   PortBitmap( currBitmap ); /* attach the current bitmap to it            */
   PortSize( currBitmap->pixWidth, currBitmap->pixHeight );
	 if ( sourceR != NULL )
       ClipRect( sourceR);

   else
	 ClipRect( &pcxPort.portRect);

   

   /* calculate the rectangle to write */
   R.Xmin = workR.Xmin;
   R.Xmax = imPtr->imWidth + workR.Xmin;
   height = pcxInfo.pcxWndo.Ymax - pcxInfo.pcxWndo.Ymin;
/*
** Begin. Unpack the byte stream into an intermediate buffer.
** This is a much simpler method when dealing with multiplanar source
** files, as it is packed into a continuous stream.
*/
    if(xlate==1)buf2 = (image *)malloc((unsigned int)imPtr->imWidth*5);
    if(xlate==1)pcxPtr = (image *)malloc((unsigned int)imPtr->imWidth*5);

    if(buf2==NULL || pcxPtr==NULL){
    free(unpkPtr);
    free(buf2);
    free(pcxPtr);
   fclose( datfile );
  return 0;

   }

   /* one  'y' scan line at a time. */
   for( y = 0; y <= height; y++) {

      /* Adjust buffer pointer for data stream. */
	wrkPtr = unpkPtr + sizeof(imageHeader);

      /* set rectangle up */
      R.Ymin = R.Ymax = workR.Ymin + y; 
	R.Ymax = R.Ymin + 1;  /* write image one scan line */


      /* build the raster lines */
      for( x = 0;  x < lineCnt;   ) {

	   /* get the byte in question */
	   unwork= (Byte) getc(datfile);

	   if( (unwork & PACKED) == PACKED ) {

		/* calculate byte value */
            z = unwork & MASK;

            unwork2 = (Byte) getc(datfile);

		/* unpack info to buffer */
            for( i = 0; i < z; i++ ) 
               *wrkPtr++ = unwork2;

		x += z;
         }
         else { /* not packed */
		*wrkPtr++ = unwork;
            x++;
         }

      }  /* next picture dot in x direction */

	 if(xlate==1 && colortable==0){
	 XlateImage(unpkPtr,buf2,currBitmap->pixBits,currBitmap->pixPlanes,NULL);
	   WriteImage( &R,buf2 );
	 }

	 else if(xlate==1 && colortable==1){
	  for( j=0; j<imPtr->imWidth; j++ )
	  pcxPtr[j] = clrmap[unpkPtr[j+sizeof(imageHeader)]];


	 for(j=0;j<imPtr->imWidth;j++)
	 unpkPtr[j+sizeof(imageHeader)]=pcxPtr[j];
	  //add sizeof imageHeader (12 bytes) for WriteImage buf record in real mode

	 XlateImage(unpkPtr,buf2,currBitmap->pixBits,currBitmap->pixPlanes,NULL);
	   WriteImage( &R,buf2 );
	 }

	 else{
	/* write to current ports bitmap */
      WriteImage( &R,unpkPtr );
     }
   }  /* next multiplane raster line */

   if(xlate==1){
   free(buf2);
   free(pcxPtr);
    }
   if(colortable==1)delete clrmap;
   /* DONE ! */
   SetPort(currPort);   /* restore original port */
   free( unpkPtr ); /* unpacked line */
   fclose( datfile );
 ShowCursor();
   return 1;
}

int writ_byts( short wb_count, register Byte wb_last, FILE *wb_datfile );


/*
This function copies the pixel image from the currnet port and bitmap
to the specified PCX file.
*/

int SEQ WritePCX( char *fname, rect *sourceR, palData RGBColors[] )
{
   grafPort       *currPort, pcxPort;  
   grafMap        *currBitmap;
   pcxHead        pcxInfo;
   FILE           *datfile;
   int            i, y, p, rowCnt, count, align;
   long           imBytes;
   rect           workR, R;
   Byte           next, curr, last;
   Word           tmp_curr;
   Byte           *pkPtr, *bufPtr, *wrkPtr;
   imageHeader    *imPtr;


   GetPort(&currPort);
   currBitmap = currPort->portMap;

   /* check if the source rect pointer is null */
   if ( sourceR == NULL )
      DupRect( &(currPort->portRect), &workR ); /* use current port's rect */
   else
      DupRect( sourceR, &workR );

   /* must do all work in global coordinates */
   if ( currPort->portFlags & pfVirtual ) /* using virtual coordinates? */
      Vir2GblRect( &workR );
   else
      Lcl2GblRect( &workR );    /* in case we are using local coordinates */

   InitPort( &pcxPort );   /* use a fresh port with everything in global */


/*
** Calculate one raster line of the rectangle to write
*/ 
   R.Xmin= workR.Xmin;
   R.Xmax= workR.Xmax; 
   R.Ymin= 0;
   R.Ymax= 1;

/*
** allocate an image buffer for one raster line
*/
   imBytes = ImageSize(&R);   /* get size to malloc */
   imBytes += 2;              /* extra so we can 'look ahead' */
   if( ( pkPtr = (Byte *)( malloc( (unsigned int)imBytes ) ) )  == NULL )
      return 0;

/*
** read the first raster line into an image buffer
*/
   ReadImage(&R, pkPtr);
   imPtr = (imageHeader *)pkPtr;       /* pointer to MW image header */

/*
** Fill in the PCX File Header
*/
   pcxInfo.pcxManf = 10;         
   pcxInfo.pcxVers = 5;         
   pcxInfo.pcxEncode = 1;     
   pcxInfo.pcxBits = (Byte)imPtr->imBits;
   pcxInfo.pcxPlanes = (Byte)imPtr->imPlanes;    
   pcxInfo.pcxRowBytes = imPtr->imRowBytes;
   pcxInfo.pcxWndo = workR;         
   pcxInfo.pcxResX = currBitmap->pixResX;         
   pcxInfo.pcxResY = currBitmap->pixResY;          
   pcxInfo.pcxRsvd = 0;         

/*
** Fill in the digital color palette
*/

   for( i=0; i<16; i++ )
   {
      /* store the values into header */
      /* PCX specifies 8 bit color values */
      p = i*3;
      pcxInfo.pcxColor[p+0]= (Byte)(RGBColors[i].palRed/256); 
      pcxInfo.pcxColor[p+1]= (Byte)(RGBColors[i].palGreen/256); 
      pcxInfo.pcxColor[p+2]= (Byte)(RGBColors[i].palBlue/256); 
   }

/*
** Open the file
*/
   if( (datfile=fopen( fname,"w+b")) == NULL)    /* binary mode write */
      return 0;

/*
** Write Header
*/
   if( fwrite(&pcxInfo, sizeof(pcxHead), 1, datfile) == 0)  {
      fclose(datfile);
      return 0;
   }

/*
** Packing and Writing Screen to PCX file 
*/
   /* one  'y' scan line at a time. */ 
   for( y = 0; y <= workR.Ymax-workR.Ymin; y++)
   {
      /* set-up next raster line */
      R.Ymin= workR.Ymin+y;
      R.Ymax= R.Ymin+1;

      ReadImage(&R, pkPtr);

      /* Adjust buffer pointer for data stream. */
      bufPtr = pkPtr + sizeof(imageHeader);

      /* shift all image data left by imAlign pixels */
      align = imPtr->imAlign * imPtr->imBits;
      count = imPtr->imRowBytes * imPtr->imPlanes;
      wrkPtr = bufPtr;

      /* if shifting less than a byte, use this loop */
      if( align > 0 && align < 8 ) {

         align = 8 - align; /* convert left shift to right shift */

         for( i = 0; i < count; i++) {
            curr = *wrkPtr;                            
            next = *(wrkPtr+1);                        
            tmp_curr = (curr << 8) + next;  /* make word */       
            *wrkPtr++ = (Byte) ( tmp_curr >> align );
         }

      }

      /* if shifting more than a byte, use this loop */
      if( align >= 8 ) {

         align -= 8;        /* implied byte shift included */
         align = 8 - align; /* convert left shift to right shift */

         for( i = 0; i < count; i++ ) {
            curr = *(wrkPtr+1);                            
            next = *(wrkPtr+2);           
            tmp_curr = (curr << 8) + next; /* make word */         
            *wrkPtr++ = (Byte) ( tmp_curr >> align );
         }

      }

         
      /* do all planes */
      for ( p = 0; p < imPtr->imPlanes; p++ ) {

         wrkPtr = bufPtr;

         count = 1;
         last = *wrkPtr;      /* get the byte in question */
         
         /* write out all bytes in row (raster) */
         for( rowCnt = 1; rowCnt < imPtr->imRowBytes; rowCnt++) {

            curr = *(++wrkPtr);  /* get the next byte */
         
            if (curr == last) {

               count++;

               if (count == 63) {   /* count can only be six bits long (63) */
                  if ( writ_byts( count, last, datfile) )
                     return 0;
                  count = 0;   
               }

            }
            else  { 
               if (count) {
                  if ( writ_byts( count, last, datfile) )
                     return 0;
               } 
               last = curr;

               count = 1;

            }

         }    /* for all bytes in row (raster) */
   
         if (count) {
            if ( writ_byts( count, last, datfile) )
               return 0;
         }

         bufPtr += imPtr->imRowBytes;  /* point to beginning of next plane */
              
      }    /* for loop for all planes */

   }    /* for loop all of raster lines */

   SetPort(currPort);   /* restore original port */
   free( pkPtr );       /* free packing buffer */


   /* if 256 color modes, write out the analog palette */
   if( QueryColors() != 255 || pcxInfo.pcxVers < 5  ) {
      fclose( datfile );
      return 1; 
   }

   /* write 'magic number', signature byte */
   putc( 12, datfile );

   for( i = 0; i < 256; i++ ) {

      /* append scaled values to file */
      /* PCX specifies 8 bit color values */
      putc( RGBColors[i].palRed/256,   datfile );
      putc( RGBColors[i].palGreen/256, datfile );
      putc( RGBColors[i].palBlue/256,  datfile );

   }

/*
** DONE !
*/

   fclose( datfile );
   return 1;

}


int writ_byts( short wb_count, register Byte wb_last, FILE *wb_datfile )
{
   if(wb_count) {

      if( (wb_count == 1) && (PACK != (PACK&wb_last)) ) {
         if( EOF == putc( (int)wb_last, wb_datfile))
            return 1;  /* disk write error */
      }

      else  { /* pack bytes */

         if( EOF == putc( (int)PACK | wb_count, wb_datfile))
            return 1;  /* disk write error */
         if( EOF == putc( (int)wb_last, wb_datfile))    
            return 1;  /* disk write error */
      }

   }

   return 0;

} /* end of writ_byts */

   
