//--------------------------------------------------------//
//                                                        //
//   File:    WRTTIF.CPP                                  //
//                                                        //
//   Desc:    Code to output a TIFF file from a           //
//            a ReadImage Bitmap            //
//                                                        //
//--------------------------------------------------------//
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include "metawndo.h"      /* master MetaWINDOW include file */
#include "GUIconst.h"
#include "GUIextrn.h"

#include "tif.hpp"
#include "tifcodec.hpp"
static   palData   RGBdata[256];
extern palData   PCXColors[256];
extern int meta_hd_size;      //imageHeader (12 bytes) for ReadImage buf record
extern int *clrmap;          // color map counter
void Pal_Table(void);
int Tif_Save(char *fn,rect *R);
int load_tif( char *fn,rect *wR );
int tifmain( void);

void show_core(char *txt);


//..................Tag set for color-mapped images

enum ColorMappedTagSet
{
   cmWidth,
   cmHeight,
   cmBitSamp,
   cmCompress,
   cmPhotoMet,
   cmStripOfs,
   cmStripHgt,
   cmXres,
   cmYres,
   cmResUnit,
   cmColorMap,
};

static IFDENTRY ColorMappedIFD[] =
{
   IFDENTRY( 256, tUShort,    1, 0L ),   // width
   IFDENTRY( 257, tUShort,    1, 0L ),   // height
   IFDENTRY( 258, tUShort,    1, 0L ),   // bits/samp
   IFDENTRY( 259, tUShort,    1, 0L ),   // compression
   IFDENTRY( 262, tUShort,    1, 0L ),   // photomet intrp
   IFDENTRY( 273, tULong,     1, 0L ),   // strip offsets
   IFDENTRY( 278, tUShort,    1, 0L ),   // rows/strip
   IFDENTRY( 282, tURational, 1, 0L ),   // x res
   IFDENTRY( 283, tURational, 1, 0L ),   // y res
   IFDENTRY( 296, tUShort,    1, 0L ),   // res unit
   IFDENTRY( 320, tUShort,    1, 0L ),   // color map
};

int ColorMappedCnt( void )
{
   return sizeof(ColorMappedIFD) / sizeof(IFDENTRY);
}

IFDENTRY * ColorMappedTags( void )
{
   IFDENTRY *cmt = new IFDENTRY [ ColorMappedCnt() ];
   if( cmt )
	for( int i=0; i<ColorMappedCnt(); i++ )
         cmt[i] = ColorMappedIFD[i];
   return cmt;
}

//..................Tag set for RGB images

enum RgbTagSet
{
   rgbWidth,
   rgbHeight,
   rgbBitSamp,
   rgbCompress,
   rgbPhotoMet,
   rgbStripOfs,
   rgbSampPix,
   rgbStripHgt,
   rgbXres,
   rgbYres,
   rgbResUnit,
};

static IFDENTRY RgbIFD[] =
{
   IFDENTRY( 256, tUShort,    1, 0L ),   // width
   IFDENTRY( 257, tUShort,    1, 0L ),   // height
   IFDENTRY( 258, tUShort,    1, 0L ),   // bits/samp
   IFDENTRY( 259, tUShort,    1, 0L ),   // compression
   IFDENTRY( 262, tUShort,    1, 0L ),   // photomet intrp
   IFDENTRY( 273, tULong,     1, 0L ),   // strip offsets
   IFDENTRY( 277, tUShort,    1, 0L ),   // samps/pixel
   IFDENTRY( 278, tUShort,    1, 0L ),   // rows/strip
   IFDENTRY( 282, tURational, 1, 0L ),   // x res
   IFDENTRY( 283, tURational, 1, 0L ),   // y res
   IFDENTRY( 296, tUShort,    1, 0L ),   // res unit
   IFDENTRY( 320, tUShort,    1, 0L ),   // color map
};

int RgbCnt( void )
{
   return sizeof(RgbIFD) / sizeof(IFDENTRY);
}

IFDENTRY * RgbTags( void )
{
   IFDENTRY *rgbt = new IFDENTRY [ RgbCnt() ];
   if( rgbt )
	for( int i=0; i<RgbCnt(); i++ )
         rgbt[i] = RgbIFD[i];
   return rgbt;
}

int Tif_Save(char *fn,rect *R)

{   //show_core("tif save start");
   rect sR;
   image *pkPtr,*scanln,*buf;
   int i,j,y,maxcolors,rwbytes;

   grafPort *wPort;
   DupRect( R, &sR );
   GetPort(&wPort);

   int nrows   = sR.Ymax-sR.Ymin ;
   int ncols   = sR.Xmax-sR.Xmin ;

   int nbits   = (wPort->portMap->pixBits == 24) ? 24 : 8;
   int ncolors = 2;
   if( wPort->portMap->pixBits == 24 ) ncolors = 0;
   else ncolors = 256;
    //will use 256 colors even for 16 color mode
  // rwbytes=wPort->portMap->pixPlanes*ncols;

     scanln = (image *)malloc(ncols*5);
	if( scanln == NULL ) {
	Dialog( diaAlert, "Not enough memory!", "for buffer" );
	return 0;
	 }

     buf = (image *)malloc(ncols*5);
	if( buf == NULL ) {
	free(scanln);
	Dialog( diaAlert, "Not enough memory!", "for buffer" );
	return  0;
	 }
   pkPtr = (image *) malloc(ncols*5  );
	if(pkPtr==NULL)
	{  free(scanln);
	   free(buf);
	   Dialog( diaAlert, "Not enough memory!", "for buffer" );
	   return 0;
	}

   // create and initialize a Tiff Image Definition
   TIFFIMAGEDEF idef;
   idef.imgtype = (nbits==24) ? iRGB : iColorMapped;
   idef.cmptype = (nbits==24) ? cUncompressed : cPackBits;
   idef.width = ncols;
   idef.height = nrows;
   idef.depth = nbits;
   idef.bitsamp = 8;   //alway 8 bits for us
   idef.samppix = nbits / 8;   //  8/8=1    or  24/8=3
   idef.nstrips = 1;
   idef.ncolors = ncolors;

   ReadPalette( 0, 0,ncolors-1 , RGBdata );
   //will read all 256 DAC registers
   //even in 16 color mode  I hope

   // create a TIFF header
   TIFFHEADER thdr;

   // create and initialize the tag set
   IFDENTRY *tags;
   if( idef.ncolors > 0 )
   {
	tags = ColorMappedTags();
      if( tags == 0 ) return xNOMEMORY;
	tags[cmWidth].valofs = idef.width;
	tags[cmHeight].valofs = idef.height;
	tags[cmBitSamp].valofs = idef.bitsamp;
      tags[cmCompress].valofs = idef.cmptype;
      tags[cmPhotoMet].valofs = idef.imgtype;
	tags[cmStripHgt].valofs = idef.height;
      tags[cmXres].valofs = idef.xres;
	tags[cmYres].valofs = idef.yres;
	tags[cmResUnit].valofs = idef.resunit;
	tags[cmColorMap].count = idef.ncolors * 3;
   }
   else
   {
	tags = RgbTags();
	if( tags == 0 ) return xNOMEMORY;
	tags[rgbWidth].valofs = idef.width;
      tags[rgbHeight].valofs = idef.height;
	tags[rgbBitSamp].valofs = idef.bitsamp;
      tags[rgbCompress].valofs = idef.cmptype;
	tags[rgbPhotoMet].valofs = idef.imgtype;
	tags[rgbSampPix].valofs = idef.samppix;
	tags[rgbStripHgt].valofs = idef.height;
	tags[rgbXres].valofs = idef.xres;
      tags[rgbYres].valofs = idef.yres;
	tags[rgbResUnit].valofs = idef.resunit;
   }

   // open and write the file
   FILE *f = fopen( fn, "wb" );

   // output a placeholder for the header
   if( thdr.put( f ) ) return xIOERROR;

   // output the color map if present
   if( idef.ncolors > 0 )
   {
	tags[cmColorMap].valofs = ftell( f );
	unsigned short pri;
   
	 for( i=0; i<idef.ncolors; i++ )
   {
	pri = (unsigned short)(RGBdata[i].palRed/256);
      pri <<= 8;
      fwrite( &pri, 2, 1, f );
   }
   for( i=0; i<idef.ncolors; i++ )
   {
	pri = (unsigned short)(RGBdata[i].palGreen/256);
	pri <<= 8;
      fwrite( &pri, 2, 1, f );
   }
   for( i=0; i<idef.ncolors; i++ )
   {
	pri = (unsigned short)(RGBdata[i].palBlue/256);
	pri <<= 8;
      fwrite( &pri, 2, 1, f );
   }

  }



   // output the X/Y resolutions
   int ir = (idef.ncolors > 0) ? cmXres : rgbXres;
   tags[ir].valofs = ftell( f );
   long ltmp = idef.xres;
   fwrite( &ltmp, 4, 1, f );
   ltmp = 1L;
   fwrite( &ltmp, 4, 1, f );
   ir = (idef.ncolors > 0) ? cmYres : rgbYres;
   tags[ir].valofs = ftell( f );
   ltmp = idef.yres;
   fwrite( &ltmp, 4, 1, f );
   ltmp = 1L;
   fwrite( &ltmp, 4, 1, f );

   // output the encoded bitmap
   ir = (idef.ncolors > 0) ? cmStripOfs : rgbStripOfs;
   tags[ir].valofs = ftell( f );
   TiffEncoder enc( f, &idef );
   enc.init();

   y=sR.Ymin;
   maxcolors=QueryColors()+1;

   if(maxcolors>16){   //256 color
   for( i=0; i<nrows; i++ )
   {
	 sR.Ymin = y+i;
	 sR.Ymax = y+i+1;
     ReadImage(&sR,scanln);
     //	for(b=3;b>0;b--){ // repeat this 3 times if 24 bits per pixel?
	for(j=0;j<ncols;j++)
	 buf[j]=scanln[j+meta_hd_size];
	 //minus imageHeader (12 bytes) for ReadImage buf record

	if( enc.encode( buf, ncols ) != xOKAY )
	{
	  Dialog( diaAlert, "Error with File", "encoder error" );
	 free(pkPtr);
	 free(scanln);
	 free(buf);
	 fclose( f );
	 return enc.status();
	}
   }
   }

   if(maxcolors<=16){      //16 color mode 1 bit 4 planes


    for( i=0; i<nrows; i++ )
   {
	 sR.Ymin = y+i;
	 sR.Ymax = y+i+1;

	ReadImage(&sR, pkPtr);
	XlateImage(pkPtr,buf,8,1,NULL);
	 for(j=0;j<ncols;j++)
	 scanln[j]=buf[j+meta_hd_size];  //minus imageHeader (12 bytes) for ReadImage buf record
	  if( enc.encode( scanln, ncols ) != xOKAY )
	  {
	  Dialog( diaAlert, "Error with File", "encoder error" );
	  free(pkPtr);
	  free(scanln);
	  free(buf);
	  fclose( f );
	  return 0;   // big error
	  }
     }
 }//end of 16 color
    // finished!
   free(scanln);
   free(buf);
   free(pkPtr);

  enc.term();

   // output the IFD
   thdr.ifdofs = ftell( f );
   short ntags = (ncolors > 0) ?
		     ColorMappedCnt() : RgbCnt();
   fwrite( &ntags, 2, 1, f );
   for( i=0; i<ntags; i++ )
	tags[i].put( f );
   ltmp = 0L;
   fwrite( &ltmp, 4, 1, f );

   // rewrite the updated header
   fseek( f, 0L, SEEK_SET );
   thdr.put( f );

   // finished!
   delete tags;
   fclose( f );
   //show_core("tif save end");
   return 1;
}

int load_tif( char *fn,rect *wR )
{
	  //show_core("tif show start");
	 imageHeader    *imPtr;
	 image *buf2,*pkPtr,*buf,*scanln;
	 grafPort *wPort;
	 rect sR,box;
	 int j,i,colortable=0;
	 meta_hd_size=(sizeof(imageHeader));

   // open file
   FILE *ftif = fopen( fn, "rb" );
   if( ftif == 0 ){
   fclose(ftif);
   Dialog( diaAlert, "Tiff file not found",NULL);
   return 0;
   }


   // read header
   TIFFHEADER tif;
   if( tif.get( ftif ) != 0 )	{
    fclose(ftif);
    Dialog( diaAlert, "Error reading TIFF file header",NULL);
    return 0;
   }



   // validate file
   if( ! tif.isvalid() ){
   fclose(ftif);
   Dialog( diaAlert, "Specified file not a TIFF file",NULL);
   return 0;
   }


   // create and read an image definition structure
   fseek( ftif, tif.ifdofs, SEEK_SET );
   TIFFIMAGEDEF idef;
   int retv = idef.get( ftif );
   if( retv == xUNSUPPORTED )retv=99;
   if( retv == xBADPARAM ) retv=99;
   if( retv == xNOMEMORY )retv=99;
   if( retv != xOKAY )retv=99;

   if( retv==99 ){
    fclose(ftif);
    Dialog( diaAlert, "TIFF file contains unsupported features","or Out of memory reading TIFF file" );
    return 0;
   }

 if(retv!=99){
   if(idef.ncolors>0){
	 for( i=0; i<idef.ncolors; i++ )
   {
     RGBdata[i].palRed=idef.colors[i].red;
     RGBdata[i].palGreen=idef.colors[i].grn;
     RGBdata[i].palBlue=idef.colors[i].blu;
     RGBdata[i].palRed   *= 256;
     RGBdata[i].palGreen *= 256;
     RGBdata[i].palBlue  *= 256;
    }



   WritePalette( 0, 0,idef.ncolors-1 , RGBdata );
 }
  int pxbytes = (idef.depth <= 8) ? 1 : 3;   //3=24 bits pre pixel
  int rwbytes = pxbytes * idef.width;
  int ncols   = idef.width;
  int nrows   = idef.height;

 if(ncols>1024){
  fclose(ftif);
  Dialog( diaAlert, "Not enough memory!",NULL );
  return 0;
   }


  scanln = (image *)malloc(rwbytes*5);
  if( scanln == NULL ) {
  fclose(ftif);
  Dialog( diaAlert, "Not enough memory!", "for scanline" );
  return 0;
   }

  buf = (image *)malloc(rwbytes*5);
  if( buf == NULL ) {
  free(scanln);
  fclose(ftif);
  Dialog( diaAlert, "Not enough memory!", "for buf" );
  return 0;
   }
  buf2 = (image *)malloc(rwbytes*5);
  if( ( pkPtr = (image  *)( malloc( rwbytes*5 ) ) )  == NULL ){
  fclose(ftif);
  free(pkPtr);
  free(scanln);
  free(buf);
  return 0;
   }
    if(idef.ncolors>0){

      //use this for color mapping
	 for( i=0; i<256; i++ )
	 {
	    PCXColors[i].palRed = RGBdata[i].palRed;
	    PCXColors[i].palGreen = RGBdata[i].palGreen;
	    PCXColors[i].palBlue = RGBdata[i].palBlue;
	 }
   }
   
     PenColor(0);
     GetPort(&wPort);
	 if ( wR != NULL )  DupRect( wR, &box );

	if(wR==NULL)FillRect(&wPort->portRect,1);
	 else FillRect(&box,1);


   int maxcolors=QueryColors()+1;

   SetRect(&sR,0,0,ncols,1);

   ReadImage(&sR,buf); //read imageHeader Info first

	 if(wR!=NULL)  //used to display image in box rect
   SetRect(&sR,box.Xmin,box.Ymin,box.Xmin+ncols,box.Ymin+1);


   TiffDecoder tifd( ftif, &idef );
   tifd.init();

 if((maxcolors>16) ||  ((idef.ncolors==0)&&(wPort->portMap->pixBits == 24)) )
 {  // 256 color   or 24 bit?

	for( i=0; i<nrows; i++ )
	{

	 sR.Ymin =i;
	 sR.Ymax =i+1;
	 //sR.Ymin +=1;   doesn't work for some reason
	 //sR.Ymax +=1;

	if( tifd.decode( scanln, ncols ) != xOKAY )
	   break;
	 for(j=0;j<ncols;j++)
	 buf[j+meta_hd_size]=scanln[j];     //add imageHeader (12 bytes) for WriteImage buf record

	 WriteImage(&sR,buf);


	}

  }//end of 256 color

 if(maxcolors<=16){
		if(idef.ncolors>16) {
	     //ShowCursor();
	   //int  err= Dialog( diaNull, "Convert 256 Color Graphic file ","to current video mode" );
	     //HideCursor();
	//if(err!=0)
	Pal_Table();colortable=1; //color table mapping
	 }
   //ReadImage(&sR, pkPtr);

   imPtr = (imageHeader *)pkPtr;       // pointer to MW image header
   imPtr->imWidth    = ncols;
   imPtr->imHeight   = 1;
   imPtr->imAlign    = 0;
   imPtr->imBits=8;
   imPtr->imPlanes=1;
   imPtr->imRowBytes=rwbytes; //change for 24 bits per pixel 3 planes?


   for( i=0; i<nrows; i++ )
   {

	 sR.Ymin =i;
	 sR.Ymax =i+1;

	 //sR.Ymin +=1;   doesn't work for some reason
	 //sR.Ymax +=1;

	if( tifd.decode( buf, ncols ) != xOKAY )
         break;


	   if(colortable==1){        //color mapping
	  for( j=0; j<ncols; j++ )
	  buf2[j] = clrmap[buf[j]];

	  for(j=0;j<ncols;j++)
	   pkPtr[j+meta_hd_size]=buf2[j];     //add imageHeader (12 bytes) for WriteImage buf record

     }

     else{     //no color mapping
	for(j=0;j<ncols;j++)
	 pkPtr[j+meta_hd_size]=buf[j];     //add imageHeader (12 bytes) for WriteImage buf record
	}
	memset(buf2,0x00,sizeof(buf2));
     XlateImage(pkPtr,buf2,1,4,NULL);
	WriteImage(&sR,buf2);

       

     }
   }//end of 16 color mode


   fclose(ftif);
   tifd.term();


   free(buf2);
   free(pkPtr);
   free(scanln);
   free(buf);
  if(colortable==1) delete clrmap;

   //show_core("tif show end");
   return 1;
  }//end of revt!=99
    return 0;
}


int tifmain( void)
{
	char     xfile[80];
	 rect wR;

	strcpy( xfile,"*.tif" );
	if ( Dialog( diaFile, "TIF files", xfile ) == 0 )
		return 0;

	 char *p = strchr( xfile, '.' );
	 if( p == 0 ) strcat( xfile, ".tif" );
     HideCursor();

  int err= load_tif(xfile,NULL );
   if(err==0){
   ShowCursor();
   return 0;
   }
   ShowCursor();
   return 1;
}

