//--------------------------------------------------------//
//                                                        //
//   File:    WRTWPG.CPP                                  //
//                                                        //
//   Desc:    Code to output a WPG file from a            //
//            a VGA Display or a Memory Bitmap            //
//                                                        //
//--------------------------------------------------------//

#include "wpg.hpp"

#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "ctype.h"
#include "metawndo.h"      /* master MetaWINDOW include file */
#include "GUIconst.h"
#include "GUIextrn.h"
//#include "vgapal.h"
extern int meta_hd_size;
void exit_pgm( char *msg );
extern "C"{
extern int cdecl iscale( int i, int n, int d );
}
//use this if in 32 bit mode
/*
int iscale( int i, int n, int d )
{
     i= (i * n);
     n= i / d;
     return (n);
}
*/
 int WPG_Save(char *filename,rect *R)

{

     palData   RGBdata[256];
     image *pkPtr;
     rect sR;
   grafPort *wPort;
   GetPort(&wPort);
   DupRect( R, &sR );


   int i;
   int nrows   = sR.Ymax-sR.Ymin+1 ;
   int ncols   = sR.Xmax-sR.Xmin+1 ;
   int maxcolors = 256;//QueryColors() +1;
   int nbits=8;//wPort->portMap->pixBits;
   int nclrs = maxcolors;

   image *scanln = (image *)malloc((unsigned int)ncols*5);
      if( scanln == NULL ) {
	 Dialog( diaAlert, "Not enough memory!", "for scanline" );
	return 0;
   }
   image *buf = (image *)malloc((unsigned int)ncols*5);
	if( buf == NULL ) {
	   free(scanln);
	   Dialog( diaAlert, "Not enough memory!", "for buf" );
      return  0;
   }

 // file header
   WPGHEADER wpg( 22 );

   // bitmap header
   WPGBMHEADER hdr( ncols, nrows, nbits );

   // begin WPG data
   int wpuW = iscale( ncols, 1200, hdr.bmxres );
   int wpuH = iscale( nrows, 1200, hdr.bmyres );
   WPGBEGIN bgn( wpuW, wpuH );

   // the four object tags
   WPGTAG bgntag( wpgBegin, 6 );
   WPGTAG clrtag( wpgColorMap, nclrs*3+4 );
   WPGTAG maptag( wpgBitmap, 999999L );
   WPGTAG endtag( wpgEnd, 0 );

   // Write the WPG file
   FILE *f = fopen( filename, "wb" );

   // file header
   wpg.put( f );

   // begin WPG data
   bgntag.put( f, 0 );
   bgn.put( f );

   // define color map
   clrtag.put( f, 0 );
   int ibgn = 0;
   fwrite( &ibgn, 2, 1, f );
   fwrite( &nclrs, 2, 1, f );
       ReadPalette( 0, 0,QueryColors() , RGBdata );

	for( i=0; i<maxcolors; i++ )
   {

      fputc((char) (RGBdata[i].palRed/256),f );
      fputc((char) (RGBdata[i].palGreen/256),f);
      fputc( (char)(RGBdata[i].palBlue/256), f);

    }
   // bitmap header and data
   long maptagofs = ftell( f );
   maptag.put( f, 4 );

   long mapofs1 = ftell( f );
   hdr.put( f );
   WpgEncoder enc( f, hdr );
   enc.init();
   int j;
  int fg=QueryColors()+1;

  if(fg>16){

     for( i=R->Ymin; i<nrows; i++ )
   {
       sR.Ymin = i;
       sR.Ymax = i+1;
     ReadImage(&sR,scanln);

      for(j=0;j<ncols;j++)
       buf[j]=scanln[j+meta_hd_size];  //minus imageHeader (12 bytes) for ReadImage buf record

       enc.encode( buf, ncols );
   }
  }
	  if(fg<=16){

   if( ( pkPtr = (image *)( malloc( (unsigned int) ncols*5 ) ) )  == NULL )
   {
	   free(scanln);
	   free(buf);
	   return 0;
	}
    for( i=0; i<nrows; i++ )
   {
	 sR.Ymin = i;
	 sR.Ymax = i+1;
	ReadImage(&sR, pkPtr);
	XlateImage(pkPtr,buf,8,1,NULL);
	 for(j=0;j<ncols;j++)
	 scanln[j]=buf[j+meta_hd_size];  //minus imageHeader (12 bytes) for ReadImage buf record
	 enc.encode( scanln, ncols );
    
    }
   }
   if(fg<=16) free(pkPtr);



   enc.term();
   long mapofs2 = ftell( f );

   // end WPG data
   endtag.put( f, 0 );

   // update bitmap tag length field
   fseek( f, maptagofs, SEEK_SET );
   maptag.reclength = mapofs2 - mapofs1;
   maptag.put( f, 4 );

   int retv = ferror( f );
   free(scanln);
   free(buf);
   fclose( f );
    return retv;
}

void read_bm( FILE * f, WPGBMHEADER * h )
{
    imageHeader    *imPtr;
    image *pkPtr,*buf2;
     grafPort *wPort;

     rect sR;
    int ncols=h->bmwidth;
    SetRect(&sR,0,0,ncols,1);   //need image size for ReadImage
   // scan line buffer for encoder

   image *scanln = (image *)malloc((unsigned int)ncols*5);
      if( scanln == NULL ) {
	Dialog( diaAlert, "Not enough memory!", "for scanline" );
	return ;
   }
   image *buf = (image *)malloc((unsigned int)ncols*5);
	if( buf == NULL ) {
	free(scanln);
	Dialog( diaAlert, "Not enough memory!", "for buf" );
	return ;
   }

	  GetPort(&wPort);
	   PenColor(0);
	  FillRect(&wPort->portRect,1);



   WpgDecoder d( f, h->bmdepth, h->bmwidth, h->bmheight );
   d.init();
   int j,i;

    ReadImage(&sR,buf);
   int fg=QueryColors()+1;

   if(fg>16){
   for( i=0; i<h->bmheight; i++ )
   {
    d.decode( scanln, ncols );

	 for(j=0;j<ncols;j++)
       buf[j+meta_hd_size]=scanln[j];     //add imageHeader (12 bytes) for WriteImage buf record

	 sR.Ymin = i;
	 sR.Ymax = i+1;
	WriteImage(&sR,buf);
     }

   }
	  if(fg<=16){

  buf2 = (image *)malloc((unsigned int)ncols*5);

   if( ( pkPtr = (image *)( malloc( (unsigned int) ncols*5 ) ) )  == NULL )
   {  free(buf2);
	free(scanln);
	free(buf);
	return ;
     }
/*
** read the first raster line into an image buffer
*/
   ReadImage(&sR, pkPtr);
   imPtr = (imageHeader *)pkPtr;       // pointer to MW image header
   imPtr->imWidth    = ncols;
   imPtr->imHeight   = 1;
   imPtr->imAlign    = 0;
   imPtr->imBits=8;
   imPtr->imPlanes=1;
   imPtr->imRowBytes=ncols*h->bmdepth; //?

     for( i=0; i<h->bmheight; i++ )
   {

	d.decode( buf, ncols );
	 sR.Ymin = i;
	 sR.Ymax = i+1;
	 for(j=0;j<ncols;j++)
	 pkPtr[j+meta_hd_size]=buf[j];     //add imageHeader (12 bytes) for WriteImage buf record

	XlateImage(pkPtr,buf2,1,4,NULL);


	WriteImage(&sR,buf2);
    }
     }
     if(fg<=16){
     free(buf2);
     free(pkPtr);
     }



   d.term();
 free(scanln);
 free(buf);



}

//..................Load the WPG file

void load_wpg( char *fn )
{
   // open file
   FILE *fwpg = fopen( fn, "rb" );
   if( fwpg == 0 )
      exit_pgm( "WPG file not found" );

   // read header
   WPGHEADER wpgh;
   if( fread( &wpgh, sizeof(WPGHEADER), 1, fwpg ) != 1 )
      exit_pgm( "Error reading WPG header" );

   // validate file
   if( wpgh.validate() != 0 )
      exit_pgm( "File is not a WPG file" );

   // scan file for palette and bitmap
  // RgbPalette   *imgpal;   // image palette
      palData   RGBdata[256];

   //imgpal = WpgPalette();
   WPGTAG tag;
   while( ! tag.get( fwpg ) )
   {
      WPGBMHEADER bmh;
      switch( tag.rectype )
      {
	 case wpgBitmap :
	      bmh.get( fwpg );
	       read_bm( fwpg, &bmh );
              break;

	 case wpgColorMap :

	    /*  imgpal = WpgPalette( fwpg );
	for(int i=0;i<imgpal->ncolors;i++){
      RGBdata[i].palRed=imgpal[i].colors->red;
      RGBdata[i].palGreen=imgpal[i].colors->grn;
      RGBdata[i].palBlue= imgpal[i].colors->blu;
      RGBdata[i].palRed   *= 256;
      RGBdata[i].palGreen *= 256;
      RGBdata[i].palBlue  *= 256;
       }
       */
	int c1, cc;
        char buf[25];
   fread( &c1, 2, 1, fwpg );
   fread( &cc, 2, 1, fwpg );
   //if( (c1 < 0) || (cc < 0) )break;

   // total colors
   //int nc = cc ;//+ c1;
	 for(int i=0; i<cc; i++ )
   {
      RGBdata[i].palRed   = fgetc(fwpg);
      RGBdata[i].palGreen = fgetc(fwpg);
      RGBdata[i].palBlue  = fgetc(fwpg);

      /* scale from 8 to 16 bit values */
      RGBdata[i].palRed   *= 256;
      RGBdata[i].palGreen *= 256;
      RGBdata[i].palBlue  *= 256;
	}

       WritePalette( 0, 0,cc-1, RGBdata );

	      break;

         default :
              fseek( fwpg, tag.reclength, SEEK_CUR );
              break;
      }
   }
   fclose( fwpg );
   //delete imgpal;
}

int WPGmain( void)
{
	char     pcxfile[80];
	 rect wR;

	strcpy( pcxfile,"*.wpg" );
	if ( Dialog( diaFile, "wpg files", pcxfile ) == 0 )
		return 0;

		 char *p = strchr( pcxfile, '.' );
	 if( p == 0 )
	    strcat( pcxfile, ".wpg" );
     HideCursor();
   
   load_wpg(pcxfile );


   ShowCursor();

   return 1;
}

